@@ -5,7 +5,7 @@
  *
  * @author Andreas Åkre Solberg, UNINETT AS. <andreas.solberg@uninett.no>
  * @package simpleSAMLphp
- * @version $Id: Utilities.php 2286 2010-05-07 09:14:10Z olavmrk $
+ * @version $Id: Utilities.php 2354 2010-06-23 11:20:37Z olavmrk $
  */
 class SimpleSAML_Utilities {
 
@@ -1106,38 +1106,41 @@ class SimpleSAML_Utilities {
 	 * @param $sppset       Allows to select another metadata set. (to support both saml2 or shib13)
 	 * @return A non-reversible unique identifier for the user.
 	 */
-	public static function generateUserIdentifier($idpEntityId, $spEntityId, $attributes, $idpset = 'saml20-idp-hosted', $spset = 'saml20-sp-remote') {
+	public static function generateUserIdentifier($idpEntityId, $spEntityId, array &$state, $idpset = 'saml20-idp-hosted', $spset = 'saml20-sp-remote') {
 	
 		$metadataHandler = SimpleSAML_Metadata_MetaDataStorageHandler::getMetadataHandler();
 		$idpMetadata = $metadataHandler->getMetaData($idpEntityId, $idpset);
 		$spMetadata = $metadataHandler->getMetaData($spEntityId, $spset);
 
-		if(array_key_exists('userid.attribute', $spMetadata)) {
-			$attributeName = $spMetadata['userid.attribute'];
-		} elseif(array_key_exists('userid.attribute', $idpMetadata)) {
-			$attributeName = $idpMetadata['userid.attribute'];
+		if (isset($state['UserID'])) {
+			$attributeValue = $state['UserID'];
 		} else {
-			$attributeName = 'eduPersonPrincipalName';
-		}
+			if(array_key_exists('userid.attribute', $spMetadata)) {
+				$attributeName = $spMetadata['userid.attribute'];
+			} elseif(array_key_exists('userid.attribute', $idpMetadata)) {
+				$attributeName = $idpMetadata['userid.attribute'];
+			} else {
+				$attributeName = 'eduPersonPrincipalName';
+			}
 
-		if(!array_key_exists($attributeName, $attributes)) {
-			throw new Exception('Missing attribute "' . $attributeName . '" for user. Cannot' .
-			                    ' generate user id.');
-		}
+			if(!array_key_exists($attributeName, $attributes)) {
+				throw new Exception('Missing attribute "' . $attributeName . '" for user. Cannot' .
+					' generate user id.');
+			}
 
-		$attributeValue = $attributes[$attributeName];
-		if(count($attributeValue) !== 1) {
-			throw new Exception('Attribute "' . $attributeName . '" for user did not contain exactly' .
-			                    ' one value. Cannot generate user id.');
-		}
+			$attributeValue = $attributes[$attributeName];
+			if(count($attributeValue) !== 1) {
+				throw new Exception('Attribute "' . $attributeName . '" for user did not contain exactly' .
+					' one value. Cannot generate user id.');
+			}
 
-		$attributeValue = $attributeValue[0];
-		if(empty($attributeValue)) {
-			throw new Exception('Attribute "' . $attributeName . '" for user was empty. Cannot' .
-			                    ' generate user id.');
+			$attributeValue = $attributeValue[0];
+			if(empty($attributeValue)) {
+				throw new Exception('Attribute "' . $attributeName . '" for user was empty. Cannot' .
+					' generate user id.');
+			}
 		}
 
-
 		$secretSalt = self::getSecretSalt();
 
 		$uidData = 'uidhashbase' . $secretSalt;
